# -*- coding: utf-8 -*-
from django import forms
from django.core.exceptions import ValidationError

try:
    from django.utils.translation import ugettext_lazy as _
except ImportError:
    from django.utils.translation import gettext_lazy as _


class PostalCodeForm(forms.Form):
    postal_code = forms.CharField(label=_("Postal Code"))

    def clean_postal_code(self):
        postal_code = self.cleaned_data["postal_code"]
        if not postal_code.isdigit():
            raise ValidationError(
                _("Please enter a valid postal code, only numbers are allowed"),
                code="invalid",
                params={"value": postal_code},
            )
        if len(postal_code) != 5:
            raise ValidationError(
                _("A valid postal code must be 5 characters long"),
                code="invalid",
                params={"value": postal_code},
            )
        return postal_code


class PostalCodeSearchForm(forms.Form):
    postal_code = forms.CharField(label=_("Postal Code"))

    def clean_postal_code(self):
        postal_code = self.cleaned_data["postal_code"]
        if not postal_code.isdigit():
            raise ValidationError(
                _("Please enter a valid postal code, only numbers are allowed"),
                code="invalid",
                params={"value": postal_code},
            )
        return postal_code



# -*- coding: utf-8 -*-
from django.db import models

try:
    from django.utils.translation import ugettext_lazy as _
except ImportError:
    from django.utils.translation import gettext_lazy as _

from model_utils.models import TimeStampedModel


D_ZONA_CHOICES = (
    ("Urbano", "Urbano"),
    ("Rural", "Rural"),
    ("Semiurbano", "Semiurbano"),
)

C_TIPO_ASENTA_CHOICES = (
    ("32", "Villa"),
    ("37", "Zona industrial"),
    ("27", "Poblado comunal"),
    ("20", "Finca"),
    ("09", "Colonia"),
    ("38", "Ampliación"),
    ("10", "Condominio"),
    ("34", "Zona federal"),
    ("18", "Exhacienda"),
    ("30", "Residencial"),
    ("02", "Barrio"),
    ("47", "Zona militar"),
    ("12", "Conjunto habitacional"),
    ("46", "Zona naval"),
    ("23", "Granja"),
    ("40", "Puerto"),
    ("01", "Aeropuerto"),
    ("15", "Ejido"),
    ("04", "Campamento"),
    ("45", "Paraje"),
    ("29", "Ranchería"),
    ("31", "Unidad habitacional"),
    ("24", "Hacienda"),
    ("28", "Pueblo"),
    ("11", "Congregación"),
    ("33", "Zona comercial"),
    ("22", "Gran usuario"),
    ("16", "Estación"),
    ("26", "Parque industrial"),
    ("48", "Rancho"),
    ("21", "Fraccionamiento"),
    ("17", "Equipamiento"),
)


C_ESTADO_CHOICES = (
    ("26", "Sonora"),
    ("10", "Durango"),
    ("03", "Baja California Sur"),
    ("04", "Campeche"),
    ("16", "Michoacán de Ocampo"),
    ("31", "Yucatán"),
    ("19", "Nuevo León"),
    ("12", "Guerrero"),
    ("11", "Guanajuato"),
    ("25", "Sinaloa"),
    ("15", "Ciudad de México"),
    ("29", "Tlaxcala"),
    ("13", "Hidalgo"),
    ("08", "Chihuahua"),
    ("24", "San Luis Potosí"),
    ("27", "Tabasco"),
    ("23", "Quintana Roo"),
    ("28", "Tamaulipas"),
    ("14", "Jalisco"),
    ("05", "Coahuila de Zaragoza"),
    ("09", "Ciudad de México"),
    ("21", "Puebla"),
    ("06", "Colima"),
    ("02", "Baja California"),
    ("32", "Zacatecas"),
    ("01", "Aguascalientes"),
    ("22", "Querétaro"),
    ("30", "Veracruz de Ignacio de la Llave"),
    ("20", "Oaxaca"),
    ("17", "Morelos"),
    ("18", "Nayarit"),
    ("07", "Chiapas"),
)

# Names are obtained from https://www.correosdemexico.gob.mx/SSLServicios/ConsultaCP/imagenes/Descrip.pdf


class PostalCode(TimeStampedModel):
    d_codigo = models.CharField(_("Código Postal asentamiento"), max_length=5)
    d_asenta = models.CharField(_("Nombre asentamiento"), max_length=128)
    D_mnpio = models.CharField(
        _("Nombre Municipio (INEGI, Marzo 2013)"), max_length=128
    )
    d_ciudad = models.CharField(
        _("Nombre Ciudad (Catálogo SEPOMEX)"), max_length=128, blank=True, null=True
    )
    d_CP = models.CharField(
        _("Código Postal de la Administración Postal que reparte al asentamiento"),
        max_length=128,
        blank=True,
        null=True,
    )
    c_estado = models.CharField(
        _("Clave Entidad (INEGI, Marzo 2013)"), max_length=2, choices=C_ESTADO_CHOICES
    )
    c_oficina = models.CharField(
        _("Código Postal de la Administración Postal que reparte al asentamiento"),
        max_length=5,
    )
    c_tipo_asenta = models.CharField(
        _("Clave Tipo de asentamiento (Catálogo SEPOMEX)"),
        max_length=2,
        choices=C_TIPO_ASENTA_CHOICES,
    )
    c_mnpio = models.CharField(_("Clave Municipio (INEGI, Marzo 2013)"), max_length=3)
    id_asenta_cpcons = models.CharField(
        _("Identificador único del asentamiento (nivel municipal)"), max_length=4
    )
    d_zona = models.CharField(
        _("Zona en la que se ubica el asentamiento (Urbano/Rural)"),
        max_length=10,
        choices=D_ZONA_CHOICES,
    )
    c_cve_ciudad = models.CharField(
        _("Clave Ciudad (Catálogo SEPOMEX)"), max_length=2, blank=True, null=True
    )

    @property
    def c_CP(self):
        return None

    @property
    def d_estado(self):
        return self.get_c_estado_display()

    @property
    def d_tipo_asenta(self):
        return self.get_c_tipo_asenta_display()

    def __str__(self):
        return "{}, {}, {}".format(self.d_codigo, self.D_mnpio, self.d_estado)



# Generated by Django 3.1.5 on 2022-01-20 23:44

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('django_postalcodes_mexico', '0003_auto_20210115_2331'),
    ]

    operations = [
        migrations.AlterField(
            model_name='postalcode',
            name='D_mnpio',
            field=models.CharField(max_length=128, verbose_name='Nombre Municipio (INEGI, Marzo 2013)'),
        ),
        migrations.AlterField(
            model_name='postalcode',
            name='c_cve_ciudad',
            field=models.CharField(blank=True, max_length=2, null=True, verbose_name='Clave Ciudad (Catálogo SEPOMEX)'),
        ),
        migrations.AlterField(
            model_name='postalcode',
            name='c_estado',
            field=models.CharField(choices=[('26', 'Sonora'), ('10', 'Durango'), ('03', 'Baja California Sur'), ('04', 'Campeche'), ('16', 'Michoacán de Ocampo'), ('31', 'Yucatán'), ('19', 'Nuevo León'), ('12', 'Guerrero'), ('11', 'Guanajuato'), ('25', 'Sinaloa'), ('15', 'Ciudad de México'), ('29', 'Tlaxcala'), ('13', 'Hidalgo'), ('08', 'Chihuahua'), ('24', 'San Luis Potosí'), ('27', 'Tabasco'), ('23', 'Quintana Roo'), ('28', 'Tamaulipas'), ('14', 'Jalisco'), ('05', 'Coahuila de Zaragoza'), ('09', 'Ciudad de México'), ('21', 'Puebla'), ('06', 'Colima'), ('02', 'Baja California'), ('32', 'Zacatecas'), ('01', 'Aguascalientes'), ('22', 'Querétaro'), ('30', 'Veracruz de Ignacio de la Llave'), ('20', 'Oaxaca'), ('17', 'Morelos'), ('18', 'Nayarit'), ('07', 'Chiapas')], max_length=2, verbose_name='Clave Entidad (INEGI, Marzo 2013)'),
        ),
        migrations.AlterField(
            model_name='postalcode',
            name='c_mnpio',
            field=models.CharField(max_length=3, verbose_name='Clave Municipio (INEGI, Marzo 2013)'),
        ),
        migrations.AlterField(
            model_name='postalcode',
            name='c_oficina',
            field=models.CharField(max_length=5, verbose_name='Código Postal de la Administración Postal que reparte al asentamiento'),
        ),
        migrations.AlterField(
            model_name='postalcode',
            name='c_tipo_asenta',
            field=models.CharField(choices=[('32', 'Villa'), ('37', 'Zona industrial'), ('27', 'Poblado comunal'), ('20', 'Finca'), ('09', 'Colonia'), ('38', 'Ampliación'), ('10', 'Condominio'), ('34', 'Zona federal'), ('18', 'Exhacienda'), ('30', 'Residencial'), ('02', 'Barrio'), ('47', 'Zona militar'), ('12', 'Conjunto habitacional'), ('46', 'Zona naval'), ('23', 'Granja'), ('40', 'Puerto'), ('01', 'Aeropuerto'), ('15', 'Ejido'), ('04', 'Campamento'), ('45', 'Paraje'), ('29', 'Ranchería'), ('31', 'Unidad habitacional'), ('24', 'Hacienda'), ('28', 'Pueblo'), ('11', 'Congregación'), ('33', 'Zona comercial'), ('22', 'Gran usuario'), ('16', 'Estación'), ('26', 'Parque industrial'), ('48', 'Rancho'), ('21', 'Fraccionamiento'), ('17', 'Equipamiento')], max_length=2, verbose_name='Clave Tipo de asentamiento (Catálogo SEPOMEX)'),
        ),
        migrations.AlterField(
            model_name='postalcode',
            name='d_CP',
            field=models.CharField(blank=True, max_length=128, null=True, verbose_name='Código Postal de la Administración Postal que reparte al asentamiento'),
        ),
        migrations.AlterField(
            model_name='postalcode',
            name='d_asenta',
            field=models.CharField(max_length=128, verbose_name='Nombre asentamiento'),
        ),
        migrations.AlterField(
            model_name='postalcode',
            name='d_ciudad',
            field=models.CharField(blank=True, max_length=128, null=True, verbose_name='Nombre Ciudad (Catálogo SEPOMEX)'),
        ),
        migrations.AlterField(
            model_name='postalcode',
            name='d_codigo',
            field=models.CharField(max_length=5, verbose_name='Código Postal asentamiento'),
        ),
        migrations.AlterField(
            model_name='postalcode',
            name='d_zona',
            field=models.CharField(choices=[('Urbano', 'Urbano'), ('Rural', 'Rural'), ('Semiurbano', 'Semiurbano')], max_length=10, verbose_name='Zona en la que se ubica el asentamiento (Urbano/Rural)'),
        ),
        migrations.AlterField(
            model_name='postalcode',
            name='id_asenta_cpcons',
            field=models.CharField(max_length=4, verbose_name='Identificador único del asentamiento (nivel municipal)'),
        ),
    ]



# Generated by Django 3.1.5 on 2021-01-15 21:56

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('django_postalcodes_mexico', '0001_initial'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='postalcode',
            name='c_CP',
        ),
        migrations.AddField(
            model_name='postalcode',
            name='d_CP',
            field=models.CharField(blank=True, max_length=128, null=True, verbose_name='Código postal'),
        ),
        migrations.AlterField(
            model_name='postalcode',
            name='D_mnpio',
            field=models.CharField(max_length=128, verbose_name='Municipio'),
        ),
        migrations.AlterField(
            model_name='postalcode',
            name='c_cve_ciudad',
            field=models.CharField(blank=True, max_length=2, null=True, verbose_name='Asentamiento'),
        ),
        migrations.AlterField(
            model_name='postalcode',
            name='c_estado',
            field=models.CharField(choices=[('26', 'Sonora'), ('10', 'Durango'), ('03', 'Baja California Sur'), ('04', 'Campeche'), ('16', 'Michoacán de Ocampo'), ('31', 'Yucatán'), ('19', 'Nuevo León'), ('12', 'Guerrero'), ('11', 'Guanajuato'), ('25', 'Sinaloa'), ('15', 'México'), ('29', 'Tlaxcala'), ('13', 'Hidalgo'), ('08', 'Chihuahua'), ('24', 'San Luis Potosí'), ('27', 'Tabasco'), ('23', 'Quintana Roo'), ('28', 'Tamaulipas'), ('14', 'Jalisco'), ('05', 'Coahuila de Zaragoza'), ('09', 'Ciudad de México'), ('21', 'Puebla'), ('06', 'Colima'), ('02', 'Baja California'), ('32', 'Zacatecas'), ('01', 'Aguascalientes'), ('22', 'Querétaro'), ('30', 'Veracruz de Ignacio de la Llave'), ('20', 'Oaxaca'), ('17', 'Morelos'), ('18', 'Nayarit'), ('07', 'Chiapas')], max_length=2, verbose_name='Código de estado'),
        ),
        migrations.AlterField(
            model_name='postalcode',
            name='c_mnpio',
            field=models.CharField(max_length=3, verbose_name='Código de municipio'),
        ),
        migrations.AlterField(
            model_name='postalcode',
            name='c_oficina',
            field=models.CharField(max_length=5, verbose_name='Código de oficina'),
        ),
        migrations.AlterField(
            model_name='postalcode',
            name='c_tipo_asenta',
            field=models.CharField(choices=[('32', 'Villa'), ('37', 'Zona industrial'), ('27', 'Poblado comunal'), ('20', 'Finca'), ('09', 'Colonia'), ('38', 'Ampliación'), ('10', 'Condominio'), ('34', 'Zona federal'), ('18', 'Exhacienda'), ('30', 'Residencial'), ('02', 'Barrio'), ('47', 'Zona militar'), ('12', 'Conjunto habitacional'), ('46', 'Zona naval'), ('23', 'Granja'), ('40', 'Puerto'), ('01', 'Aeropuerto'), ('15', 'Ejido'), ('04', 'Campamento'), ('45', 'Paraje'), ('29', 'Ranchería'), ('31', 'Unidad habitacional'), ('24', 'Hacienda'), ('28', 'Pueblo'), ('11', 'Congregación'), ('33', 'Zona comercial'), ('22', 'Gran usuario'), ('16', 'Estación'), ('26', 'Parque industrial'), ('48', 'Rancho'), ('21', 'Fraccionamiento'), ('17', 'Equipamiento')], max_length=2, verbose_name='Código de tipo de asentamiento'),
        ),
        migrations.AlterField(
            model_name='postalcode',
            name='d_asenta',
            field=models.CharField(max_length=128, verbose_name='Asentamiento'),
        ),
        migrations.AlterField(
            model_name='postalcode',
            name='d_ciudad',
            field=models.CharField(blank=True, max_length=128, null=True, verbose_name='Ciudad'),
        ),
        migrations.AlterField(
            model_name='postalcode',
            name='d_codigo',
            field=models.CharField(max_length=5, verbose_name='Codigo Postal'),
        ),
        migrations.AlterField(
            model_name='postalcode',
            name='d_zona',
            field=models.CharField(choices=[('UR', 'Urbano'), ('RU', 'Rural'), ('SE', 'Semiurbano')], max_length=2, verbose_name='Tipo de Zona'),
        ),
        migrations.AlterField(
            model_name='postalcode',
            name='id_asenta_cpcons',
            field=models.CharField(max_length=4, verbose_name='Id Asentamiento'),
        ),
    ]



# -*- coding: utf-8 -*-
from warnings import warn

from django.http import JsonResponse

try:
    from django.utils.translation import ugettext_lazy as _
except ImportError:
    from django.utils.translation import gettext_lazy as _

from .forms import PostalCodeForm, PostalCodeSearchForm
from .models import (
    PostalCode,
)
from .utils import generateCityAreasByPostalCode, searchPostalCode


def getPostalCodeData(request, postal_code):
    if request.path.startswith("/postal-code/"):
        warn(
            "The url postal-code/ is gonna be deprecated in favor of /",
            category=DeprecationWarning,
            stacklevel=2,
        )
    postalCodeForm = PostalCodeForm({"postal_code": postal_code})
    if postalCodeForm.is_valid():
        postal_code_data = generateCityAreasByPostalCode(postal_code)
        status = 200 if postal_code_data else 404
        return JsonResponse(postal_code_data, status=status)
    return JsonResponse(postalCodeForm.errors, status=400)


def searchSimilarPostalCode(request, postal_code):
    postalCodeForm = PostalCodeSearchForm({"postal_code": postal_code})
    if postalCodeForm.is_valid():
        postal_codes_list = searchPostalCode(postal_code)
        status = 200 if postal_codes_list else 404
        return JsonResponse(postal_codes_list, status=status)
    return JsonResponse(postalCodeForm.errors, status=400)



# -*- coding: utf-8 -*-
from django.urls import path
from django.views.generic import TemplateView

from . import views


app_name = 'django_postalcodes_mexico'
urlpatterns = [
    # There are postal codes with leading zeros,
    # hence slug is prefered over int
    path(
        '<slug:postal_code>/',
        views.getPostalCodeData,
        name='get-postal-code-data'
        ),
    path(
        'postal-code/<slug:postal_code>/',
        views.getPostalCodeData,
        name='get-postal-code-data-deprecated'
        ),
    path(
        'search/<slug:postal_code>/',
        views.searchSimilarPostalCode,
        name='search-postal-code-data')
]



# Generated by Django 3.1.5 on 2021-01-15 21:49

from django.db import migrations, models
import django.utils.timezone
import model_utils.fields


class Migration(migrations.Migration):

    initial = True

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='PostalCode',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created', model_utils.fields.AutoCreatedField(default=django.utils.timezone.now, editable=False, verbose_name='created')),
                ('modified', model_utils.fields.AutoLastModifiedField(default=django.utils.timezone.now, editable=False, verbose_name='modified')),
                ('d_codigo', models.CharField(max_length=5)),
                ('d_asenta', models.CharField(max_length=128)),
                ('D_mnpio', models.CharField(max_length=128)),
                ('d_ciudad', models.CharField(blank=True, max_length=128, null=True)),
                ('c_estado', models.CharField(choices=[('26', 'Sonora'), ('10', 'Durango'), ('03', 'Baja California Sur'), ('04', 'Campeche'), ('16', 'Michoacán de Ocampo'), ('31', 'Yucatán'), ('19', 'Nuevo León'), ('12', 'Guerrero'), ('11', 'Guanajuato'), ('25', 'Sinaloa'), ('15', 'México'), ('29', 'Tlaxcala'), ('13', 'Hidalgo'), ('08', 'Chihuahua'), ('24', 'San Luis Potosí'), ('27', 'Tabasco'), ('23', 'Quintana Roo'), ('28', 'Tamaulipas'), ('14', 'Jalisco'), ('05', 'Coahuila de Zaragoza'), ('09', 'Ciudad de México'), ('21', 'Puebla'), ('06', 'Colima'), ('02', 'Baja California'), ('32', 'Zacatecas'), ('01', 'Aguascalientes'), ('22', 'Querétaro'), ('30', 'Veracruz de Ignacio de la Llave'), ('20', 'Oaxaca'), ('17', 'Morelos'), ('18', 'Nayarit'), ('07', 'Chiapas')], max_length=2)),
                ('c_oficina', models.CharField(max_length=5)),
                ('c_CP', models.CharField(blank=True, max_length=128, null=True)),
                ('c_tipo_asenta', models.CharField(choices=[('32', 'Villa'), ('37', 'Zona industrial'), ('27', 'Poblado comunal'), ('20', 'Finca'), ('09', 'Colonia'), ('38', 'Ampliación'), ('10', 'Condominio'), ('34', 'Zona federal'), ('18', 'Exhacienda'), ('30', 'Residencial'), ('02', 'Barrio'), ('47', 'Zona militar'), ('12', 'Conjunto habitacional'), ('46', 'Zona naval'), ('23', 'Granja'), ('40', 'Puerto'), ('01', 'Aeropuerto'), ('15', 'Ejido'), ('04', 'Campamento'), ('45', 'Paraje'), ('29', 'Ranchería'), ('31', 'Unidad habitacional'), ('24', 'Hacienda'), ('28', 'Pueblo'), ('11', 'Congregación'), ('33', 'Zona comercial'), ('22', 'Gran usuario'), ('16', 'Estación'), ('26', 'Parque industrial'), ('48', 'Rancho'), ('21', 'Fraccionamiento'), ('17', 'Equipamiento')], max_length=2)),
                ('c_mnpio', models.CharField(max_length=3)),
                ('id_asenta_cpcons', models.CharField(max_length=4)),
                ('d_zona', models.CharField(choices=[('UR', 'Urbano'), ('RU', 'Rural'), ('SE', 'Semiurbano')], max_length=2)),
                ('c_cve_ciudad', models.CharField(blank=True, max_length=2, null=True)),
            ],
            options={
                'abstract': False,
            },
        ),
    ]



# -*- coding: utf-8 -*-
import io
import zipfile
from django.core.management.base import BaseCommand, CommandError
import xml.etree.ElementTree as ET

from ...models import PostalCode
from ._utils import (
    get_value_from_node,
    generate_list_of_postalcode_objects,
    get_xml_postal_codes_data,
    parse_xml_postal_codes_file,
)


class Command(BaseCommand):
    help = "Creates a postal code database using the xml data from the Mexican Postal Service (Correos de Mexico)"

    def add_arguments(self, parser):

        # Named (optional) arguments
        parser.add_argument(
            "--file",
            nargs="?",
            const="CPdescarga.xml",
            type=str,
            help="Specify the xml file that contains the postal codes",
        )

    def get_function_for_processing_xml_postal_codes(self, xml_file_name):
        if xml_file_name:
            return parse_xml_postal_codes_file(xml_file_name)
        return get_xml_postal_codes_data()

    def handle(self, *args, **options):
        table_already_exist = PostalCode.objects.filter(id=1).exists()
        if table_already_exist:
            self.stdout.write(self.style.WARNING("Postal code table already exists."))
            self.stdout.write(
                self.style.WARNING(
                    "Importing postal codes again could cause data inconsistencies, if you want to import postal codes again please delete the postal code table first."
                )
            )
            return
        try:
            xml_tree = self.get_function_for_processing_xml_postal_codes(
                options["file"]
            )
        except Exception as e:
            self.stdout.write(
                self.style.ERROR(
                    "There was an error obtaining the xml zipped file from Correos de Mexico:"
                    + str(e)
                )
            )
            exit(1)
        PostalCodes = []
        self.stdout.write(
            self.style.WARNING("This process can take a few minutes, please be patient")
        )
        postalCodes = generate_list_of_postalcode_objects(xml_tree)
        self.stdout.write("Creating table...")
        PostalCode.objects.bulk_create([PostalCode(**data) for data in postalCodes])
        self.stdout.write(
            self.style.SUCCESS(
                "The postal code table has been successfully populated"
            )
        )



# -*- coding: utf-8 -*-
from .forms import PostalCodeForm
from .models import (
    PostalCode,
)


def generateCityAreasByPostalCode(postal_code):
    postal_codes = PostalCode.objects.filter(d_codigo=postal_code)
    if postal_codes:
        postal_code_data = {
            "codigoPostal": postal_code,
            "municipio": postal_codes[0].D_mnpio,
            "estado": postal_codes[0].d_estado,
            "colonias": list(map(lambda x: x.d_asenta, postal_codes)),
        }
        return postal_code_data
    return dict()


def searchPostalCode(postal_code):
    postal_codes = PostalCode.objects.filter(d_codigo__contains=postal_code).distinct(
        "d_codigo"
    )
    if postal_codes:
        postal_code_list = [postal_code.d_codigo for postal_code in postal_codes]
        return {"postal_codes": postal_code_list}
    return dict()



#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""
test_django-postalcodes-mexico
------------

Tests for `django-postalcodes-mexico` models module.
"""

import json

from django.test import TestCase
from django.urls import reverse
from mock import patch, MagicMock, Mock

from django_postalcodes_mexico.models import PostalCode


class TestDjango_postalcodes_mexico(TestCase):
    def setUp(self):
        PostalCode.objects.get_or_create(
            d_codigo="01000",
            d_asenta="San Ángel",
            D_mnpio="Álvaro Obregón",
            d_ciudad="Ciudad de México",
            d_CP="01001",
            c_estado="09",
            c_oficina="01001",
            c_tipo_asenta="09",
            c_mnpio="010",
            id_asenta_cpcons="0001",
            d_zona="Urbano",
            c_cve_ciudad="01",
        )
        # assertEquals was deprecated in Python 3.12, since we need it to run the previous tests
        # we assign it to assertEquals if it's remove
        try:
            # This will throw an error in Python 3.12
            self.deprecated = self.assertEquals
        except AttributeError:
            self.assertEquals = self.assertEqual

    def test_api_returns_postal_code_data(self):
        retrieve_postal_code_data = reverse(
            "django_postalcodes_mexico:get-postal-code-data", args=("01000",)
        )
        response = self.client.get(retrieve_postal_code_data)
        self.assertEquals(response.status_code, 200)
        postal_code_data = json.loads(response.content)
        self.assertIsInstance(postal_code_data, dict)
        self.assertIn("colonias", postal_code_data)
        self.assertIn("municipio", postal_code_data)
        self.assertIn("estado", postal_code_data)
        self.assertIsInstance(postal_code_data.get("colonias"), list)

    def test_api_returns_404_on_not_found_postal_code(self):
        retrieve_postal_code_data = reverse(
            "django_postalcodes_mexico:get-postal-code-data", args=("99999",)
        )
        response = self.client.get(retrieve_postal_code_data)
        self.assertEquals(response.status_code, 404)
        postal_code_data = json.loads(response.content)
        self.assertEquals(len(postal_code_data), 0)

    def test_api_returns_400_on_alpha_input(self):
        retrieve_postal_code_data = reverse(
            "django_postalcodes_mexico:get-postal-code-data", args=("ABCDE",)
        )
        response = self.client.get(retrieve_postal_code_data)
        self.assertEquals(response.status_code, 400)
        postal_code_data = json.loads(response.content)
        self.assertNotIn("colonia", postal_code_data)
        self.assertIn("postal_code", postal_code_data)

    def test_api_returns_400_on_long_digit_input(self):
        retrieve_postal_code_data = reverse(
            "django_postalcodes_mexico:get-postal-code-data", args=("999999999999",)
        )
        response = self.client.get(retrieve_postal_code_data)
        self.assertEquals(response.status_code, 400)
        postal_code_data = json.loads(response.content)
        self.assertNotIn("colonia", postal_code_data)
        self.assertIn("postal_code", postal_code_data)

    def tearDown(self):
        pass
